/* Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/ */

"use strict";

function onLoad123() {
  console.log("Blah Blah");
}

window.addEventListener("load", onLoad123);
