/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * globals setupPositining, runPositioningTest, sizes
 */

"use strict";

add_setup(setupPositioning);

add_task(async function test_scrollPositions() {
  await runPositioningTest(sizes.slice(0, 2));
});
