var rollupBabel7ShadowedVars = (function () {
  'use strict';

  function test () {

    {
      {
        console.log("pause here");
      }
    }
  }

  return test;

}());
//# sourceMappingURL=shadowed-vars.js.map
